using System;
using System.Collections.Generic;
using System.Text;
using System.Data;
using System.Drawing;


namespace OrthoRef
{
    public class CExtPar
    {
        public double XL;
        public double YL;
        public double ZL;
        public double Kappa;
        public double Phi;
        public double Omega;
        public int Iterations;
        public dsGCP dsGCP = new dsGCP();
        public dsCOP dsCOP = new dsCOP();

        public CExtPar()
        {
            DataRow row;
            row = dsCOP.dtbCOP.NewRow();
            dsCOP.dtbCOP.Rows.Add(row);
          
        }
        public double rmsError
        {
            get { return _rmsError(); }
        }

        private double _rmsError()
        {
            double error = 0;
            int nGCP = dsGCP.tlbGCP.Count;
            foreach (DataRow row in dsGCP.tlbGCP.Rows)
            {
                error += Math.Pow(Convert.ToDouble(row["Error"]), 2);
            }
            return Math.Sqrt(error / nGCP);
        }

        public RectangleF rctGCP()
        {
            float xmin = 0, xmax = 0, ymin = 0, ymax = 0;
            float xGCP, yGCP;
            int i = 1;
            foreach (DataRow row in dsGCP.tlbGCP.Rows)
            {
                xGCP = Convert.ToSingle(row["GCPX"]);
                yGCP = Convert.ToSingle(row["GCPY"]);
                if( i == 1)
                {
                    xmin = xGCP;
                    xmax = xGCP;
                    ymin = yGCP;
                    ymax = yGCP;
                }
                xmin = Math.Min(xmin, xGCP);
                xmax = Math.Max(xmax, xGCP);
                ymin = Math.Min(ymin, yGCP);
                ymax = Math.Max(ymax, yGCP);
                i++;
               
            }
            RectangleF rect = new RectangleF(xmin, ymin, xmax - xmin, ymax - ymin);
            return rect;
        }

        public void TodsCOP()
        {
            DataRowCollection rows = dsCOP.dtbCOP.Rows;
            DataRow row = rows[0];
            row["XL"] = XL;
            row["YL"] = YL;
            row["ZL"] = ZL;
            row["Phi"] = Phi;
            row["Omega"] = Omega;
            row["Kappa"] = Kappa;
            
        }

        public void FromdsCOP()
        {
            DataRowCollection rows = dsCOP.dtbCOP.Rows;
            DataRow row = rows[0];
            XL = (double) row["XL"];
            YL = (double) row["YL"];
            ZL = (double) row["ZL"];
            Phi = (double) row["Phi"];
            Omega = (double) row["Omega"];
            Kappa = (double) row["Kappa"];

        }

        public double Mean(string column)
        {

            double x;
            double _mean = 0;
            int n = dsGCP.tlbGCP.Count;
            foreach (DataRow row in dsGCP.tlbGCP.Rows)
            {
                x = Convert.ToDouble(row[column]);
                _mean += x;
            }
            return _mean / n;
        }
        public double[,] ConvertToMatrix()
        {
            int ncols = dsGCP.tlbGCP.Columns.Count;
            
            int[] cols = new int[ncols];
            
            for (int j = 0; j < ncols; j++)
                cols[j] = j;
            return ConvertToMatrix(cols);
        }
      
        public double[,] ConvertToMatrix(int[] columns)
        {
            int ncols = columns.GetLength(0);
            double[,] x = new double[dsGCP.tlbGCP.Count, ncols];
            int i = 0;

            foreach (DataRow row in dsGCP.tlbGCP.Rows)
            {
                for (int j = 0; j < ncols; j++)
                {
                    try
                    {
                        x[i, j] = Convert.ToDouble(row[columns[j]]);
                    }
                    catch
                    {
                        x[i, j] = 0;
                    }
                }

                i += 1;
            }
            return x;
        }

    }
}
