using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Text;
using System.Windows.Forms;
using System.Collections;
using System.Xml;
using System.IO;

using MathWorks.MATLAB.NET.Utility;
using MathWorks.MATLAB.NET.Arrays;
using Extrinsic; // Matlab function to compute intrinsic parameters



namespace OrthoRef
{
    public partial class Main : Form
    {
        public CExtPar CExtPar = new CExtPar(); // Camera Extrinsic Parameters, includes ground control point information
        public CIntPar CIntPar = new CIntPar(); // Camera Intrinsic Parameters
        public ArrayList arrOriginalImages = new ArrayList();
        public ArrayList arrCorrectedImages = new ArrayList();
        public ArrayList arrRectifiedImages = new ArrayList();
        public String strCamera;
  
        Extrinsicclass MWN = new Extrinsicclass();

        double TargetResolution;
        public double[] TargetArea = { 0, 0, 0, 0 };


        public Main()
        {
            InitializeComponent();
        }

        private void extrinsicToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ComputeExtrinsicPar frmComputeExtrinsicPar = new ComputeExtrinsicPar();
            double fl = (CIntPar.Fx + CIntPar.Fy) / 2;
            frmComputeExtrinsicPar.FocalLenght = fl;
            if (fl > 0)
            {
                frmComputeExtrinsicPar.computeToolStripMenuItem.Enabled = true;
                frmComputeExtrinsicPar.btnCompute.Enabled = true;
            }
            if (frmComputeExtrinsicPar.ShowDialog() == DialogResult.OK)
            {

                CExtPar = frmComputeExtrinsicPar.ExtrinsicParameters;
                rectifyToolStripMenuItem1.Enabled = true;
                OrientationtoolStripStatusLabel3.Text = "-  oriented";
            }
         }

        private void openToolStripMenuItem_Click(object sender, EventArgs e)
        {
            openFileDialog1.Filter = "Image files|*.BMP;*.JPG;*.TIF;*.GIF| Corrected image files | *.BMPC; *.JPGC;*.TIFC;*.GIFC";
            if (openFileDialog1.ShowDialog() == DialogResult.OK)
            {
                string[] strImages = openFileDialog1.FileNames;
                int i = 0;
                string ShortFileName;
                foreach (string file in strImages)
                {
                    ShortFileName = file.Substring(file.LastIndexOf("\\")+ 1);

                    if (file.LastIndexOf("c") + 1 != file.Length)
                    {
                        imageList1.Images.Add(Image.FromFile(file));
                        listView1.Items.Add(ShortFileName, i);
                        arrOriginalImages.Add(file);
                    }
                    else
                    {
                        imageList2.Images.Add(Image.FromFile(file));
                        listView2.Items.Add(ShortFileName, i);
                        arrCorrectedImages.Add(file);
                    }

                    i++;
                }
                if (arrOriginalImages.Count > 0)
                    this.zoomPicBox1.Image = Image.FromFile((string)arrOriginalImages[0]);
                else
                {
                    this.zoomPicBox1.Image = Image.FromFile((string)arrCorrectedImages[0]);
                    tabControl1.SelectedTab = tabCorrected;

                }
                this.zoomPicBox1.ZoomExtents();
             }
         }

        private void ExitStripMenuItem1_Click(object sender, EventArgs e)
        {
            Application.Exit();
        }

         #region Listviews events

         private void listView1_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (listView1.SelectedIndices.Count > 0)
            {
                try
                {
                    int intImgIndex = listView1.SelectedIndices[0];
                    this.zoomPicBox1.Image = Image.FromFile((string)arrOriginalImages[intImgIndex]);
                    this.zoomPicBox1.ZoomExtents();
                }
                catch
                {
                }
            }
        }

        private void listView2_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (listView2.SelectedIndices.Count > 0)
            {
                int intImgIndex = listView2.SelectedIndices[0];
                this.zoomPicBox1.Image = Image.FromFile((string)arrCorrectedImages[intImgIndex]);
                this.zoomPicBox1.ZoomExtents();
           }
        }

        private void listView3_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (listView3.SelectedIndices.Count > 0)
            {
                int intImgIndex = listView3.SelectedIndices[0];
                this.zoomPicBox1.Image = Image.FromFile((string)arrRectifiedImages[intImgIndex]);
                this.zoomPicBox1.ZoomExtents();
            }

        }

        private void listView1_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Delete)
            {
                int j = 0;
                foreach (int i in listView1.SelectedIndices)
                {
                    listView1.Items.RemoveAt(i - j); //Account for deleted items
                    imageList1.Images.RemoveAt(i - j);
                    arrOriginalImages.RemoveAt(i - j);
                    j += 1;

                }
                this.zoomPicBox1.Image = null;
            }
        }

        private void listView2_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Delete)
            {
                int j = 0;
                foreach (int i in listView2.SelectedIndices)
                {
                    listView2.Items.RemoveAt(i - j); //Account for deleted items
                    imageList2.Images.RemoveAt(i - j);
                    arrCorrectedImages.RemoveAt(i - j);
                    j += 1;

                }
                this.zoomPicBox1.Image = null;
            }
        }

        private void listView3_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Delete)
            {
                int j = 0;
                foreach (int i in listView3.SelectedIndices)
                {
                    listView3.Items.RemoveAt(i - j); //Account for deleted items
                    imageList3.Images.RemoveAt(i - j);
                    arrRectifiedImages.RemoveAt(i - j);
                    j += 1;

                }
                this.zoomPicBox1.Image = null;
            }

        }

        #endregion

         private void intrinsicToolStripMenuItem_Click(object sender, EventArgs e)
        {
            openFileDialog2.Filter = "Camera Definition Files|*.Xml";
            if (openFileDialog2.ShowDialog() == DialogResult.OK)
            {

                XmlDocument document = new XmlDocument();
                document.Load(openFileDialog2.FileName);
                if (document.DocumentElement.Name == "Cameras")
                {
                    CameraSelection frmCameraSelection = new CameraSelection();


                    foreach (XmlNode elem in document.DocumentElement)
                    {
                        frmCameraSelection.cbbCamera.Items.Add(elem.Name);
                    }
                    frmCameraSelection.document = document;
                    if (frmCameraSelection.ShowDialog() == DialogResult.OK)
                    {
                        CIntPar = frmCameraSelection.IntrinsicParameter;
                        strCamera = (string)frmCameraSelection.cbbCamera.SelectedItem;
                        undistortImagesToolStripMenuItem.Enabled = true;
                        CameratoolStripStatusLabel2.Text = "Camera: " + strCamera;
                    }

                }
                else
                {
                    MessageBox.Show("Not a valid camera definition file");
                }

            }


        }

        private void undistortImagesToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (arrOriginalImages.Count == 0)
            {
                MessageBox.Show("No images to process");
                return;
            }
    
            int i = 0;
            string ShortFileName;
            string filec;
            int[] ImageSize ={ 0, 0, 0, 0 };
            double[] IntPar = {CIntPar.Fx, CIntPar.Fy, CIntPar.Cx, CIntPar.Cy, CIntPar.K1, CIntPar.K2, CIntPar.K3, CIntPar.K4, CIntPar.K5, CIntPar.Alpha0 };
   
            ImageCorrectionParameters frmImageCorrectionParameters = new ImageCorrectionParameters();
            frmImageCorrectionParameters.rdbPerOriginal.Checked = true;
            frmImageCorrectionParameters.lblCamera.Text = "Camera: " + strCamera;
            
            if (frmImageCorrectionParameters.ShowDialog() == DialogResult.OK)
            {
                if (frmImageCorrectionParameters.rdbPerOriginal.Checked == true)
                {
                    ImageSize[1] = Convert.ToInt32(frmImageCorrectionParameters.nudOriginal.Value);
                }
                else if (frmImageCorrectionParameters.rdbCC.Checked == true)
                {
                    ImageSize[0] = Convert.ToInt32(frmImageCorrectionParameters.nudCC.Value);
                }
                else
                {
                    ImageSize[0] = Convert.ToInt32(frmImageCorrectionParameters.txbxmin.Text);
                    ImageSize[1] = Convert.ToInt32(frmImageCorrectionParameters.txbxmax.Text);
                    ImageSize[2] = Convert.ToInt32(frmImageCorrectionParameters.txbymin.Text);
                    ImageSize[3] = Convert.ToInt32(frmImageCorrectionParameters.txbymax.Text);
                }
                toolStripStatusLabel1.Text = " Processing images ";
                toolStripProgressBar1.Visible = true;
                toolStripProgressBar1.Maximum = arrOriginalImages.Count;

                this.Refresh();
                foreach (Object obj in arrOriginalImages)
                {
                    toolStripProgressBar1.Value = i;
                    MWN.intrinsic((MWNumericArray)IntPar, (MWNumericArray) ImageSize, (string)arrOriginalImages[i]);
                    // To be corrected 
                    filec = (string)arrOriginalImages[i] + "c";
                    arrCorrectedImages.Add(filec);
                    //strImages2[i] = filec;
                    imageList2.Images.Add(Image.FromFile(filec));
                    ShortFileName = filec.Substring(filec.LastIndexOf("\\") + 1);
                    listView2.Items.Add(ShortFileName, i);
                    i++;
                }
                toolStripProgressBar1.Visible = false;

                tabControl1.SelectedTab = tabCorrected;
                toolStripStatusLabel1.Text = i + " images succesfully processed.";
                this.zoomPicBox1.Image = Image.FromFile((string)arrCorrectedImages[0]);
                this.zoomPicBox1.ZoomExtents();
            }
        }


        private void rectifyToolStripMenuItem1_Click(object sender, EventArgs e)
        {
            if (arrCorrectedImages.Count == 0)
            {
                MessageBox.Show("No images to process");
                return;
            }
            

            double[] ExtPar = { CExtPar.XL, CExtPar.YL, CExtPar.ZL, CExtPar.Kappa, CExtPar.Phi, CExtPar.Omega };
            double[] IntPar = { CIntPar.Fx, CIntPar.Fy, CIntPar.Cx, CIntPar.Cy, CIntPar.K1, CIntPar.K2, CIntPar.K3, CIntPar.K4, CIntPar.K5, CIntPar.Alpha0 };
            int i = 0;
            TargetResolution = 1;
            double zLevel;
            string InterpolationMethod;


            string ShortFileName;
            string OriginalFileName;
            string RectifiedFileName;
            
            RectificationParameters frmRectificationParameters = new RectificationParameters();

            frmRectificationParameters.txbZ.Text = CExtPar.Mean("GCPZ").ToString();

            frmRectificationParameters.cbxMethod.SelectedItem = "Nearest";

            float xmin = CExtPar.rctGCP().X;
            float xmax = CExtPar.rctGCP().X + CExtPar.rctGCP().Width;
            float ymin = CExtPar.rctGCP().Y;
            float ymax = CExtPar.rctGCP().Y + CExtPar.rctGCP().Height;

            if (TargetArea[3] == 0)
            {
                frmRectificationParameters.txbxmin.Text = xmin.ToString();
                frmRectificationParameters.txbxmax.Text = xmax.ToString();
                frmRectificationParameters.txbymin.Text = ymin.ToString();
                frmRectificationParameters.txbymax.Text = ymax.ToString();
            }
            else
            {
                frmRectificationParameters.txbxmin.Text = Convert.ToString(TargetArea[0]);
                frmRectificationParameters.txbxmax.Text = Convert.ToString(TargetArea[1]);
                frmRectificationParameters.txbymin.Text = Convert.ToString(TargetArea[2]);
                frmRectificationParameters.txbymax.Text = Convert.ToString(TargetArea[3]);
            }


           
            if (frmRectificationParameters.ShowDialog() == DialogResult.OK)
            {
                zLevel = Convert.ToDouble(frmRectificationParameters.txbZ.Text);

                TargetResolution = Convert.ToDouble(frmRectificationParameters.nudResolution.Value);

                InterpolationMethod = (string) frmRectificationParameters.cbxMethod.SelectedItem;

                TargetArea[0] = Convert.ToDouble(frmRectificationParameters.txbxmin.Text);
                TargetArea[1] = Convert.ToDouble(frmRectificationParameters.txbxmax.Text);
                TargetArea[2] = Convert.ToDouble(frmRectificationParameters.txbymin.Text);
                TargetArea[3] = Convert.ToDouble(frmRectificationParameters.txbymax.Text);
                                
                toolStripStatusLabel1.Text = " Processing images ";
                toolStripProgressBar1.Visible = true;
                toolStripProgressBar1.Maximum = arrOriginalImages.Count;

                this.Refresh();

                foreach (Object obj in arrCorrectedImages)
                {
                    OriginalFileName = (string) arrCorrectedImages[i];
                    RectifiedFileName = OriginalFileName.Substring(0, OriginalFileName.LastIndexOf(".")) + "_rect.tif";
                    int index = 1;
                    while (arrRectifiedImages.Contains(RectifiedFileName))
                    {
                        RectifiedFileName = OriginalFileName.Substring(0, OriginalFileName.LastIndexOf(".")) + "_rect" + index.ToString() + ".tif";
                        index++;
                    }
                    MWN.Rectify((MWNumericArray)IntPar, (MWNumericArray)ExtPar, (MWNumericArray)TargetResolution, (MWNumericArray)TargetArea, (MWNumericArray)zLevel, OriginalFileName, RectifiedFileName, InterpolationMethod);
                    arrRectifiedImages.Add(RectifiedFileName);
                    //strImages2[i] = filec;
                    imageList3.Images.Add(Image.FromFile(RectifiedFileName));
                    ShortFileName = RectifiedFileName.Substring(RectifiedFileName.LastIndexOf("\\") + 1);
                    listView3.Items.Add(ShortFileName, i);

                    i++;
                }
                tabControl1.SelectedTab = tabRectified;

                toolStripProgressBar1.Visible = false;
                toolStripStatusLabel1.Text = i + " images succesfully rectified.";
                this.zoomPicBox1.Image = Image.FromFile((string)arrRectifiedImages[arrRectifiedImages.Count-1]);
                this.zoomPicBox1.ZoomExtents();

            }
        }

        #region ZoomPicEvents
        
        private void zoomPicBox1_MouseClick(object sender, MouseEventArgs e)
        {
            zoomPicBox1.Zoom = 1;
        }

        private void zoomPicBox1_MouseDoubleClick(object sender, MouseEventArgs e)
        {
            zoomPicBox1.ZoomExtents();
        }

        private void zoomPicBox1_Paint(object sender, PaintEventArgs e)
        {
            if (showCPToolStripMenuItem.Checked)
            {
                Pen _pen = new Pen(Color.Aqua, 2);

                float x, y;
                foreach (DataRow row in CExtPar.dsGCP.tlbGCP.Rows)
                {
                    x = Convert.ToSingle(row["GCPX"]);
                    y = Convert.ToSingle(row["GCPY"]);

                    x -= (float) TargetArea[0];
                    y -= (float) TargetArea[2];

                    x /= (float) TargetResolution;
                    y /= (float) TargetResolution;

                    e.Graphics.DrawLine(_pen, x - 5, y, x + 5, y);
                    e.Graphics.DrawLine(_pen, x, y - 5, x, y + 5);
                }
                zoomPicBox1.Invalidate();
            }
        }


        #endregion

        private void aboutToolStripMenuItem_Click(object sender, EventArgs e)
        {
         }

        private void showCPToolStripMenuItem_Click(object sender, EventArgs e)
        {
            showCPToolStripMenuItem.Checked = !showCPToolStripMenuItem.Checked;
            zoomPicBox1.Invalidate();
        }

         private void imageToolStripMenuItem_Click(object sender, EventArgs e)
        {
            DisplayImageOptions  frmDisplayImageOptions = new DisplayImageOptions();
            Type imode = typeof(InterpolationMode);
            foreach (string s in Enum.GetNames(imode))
            {
                frmDisplayImageOptions.cbbInterpolation.Items.Add(s);
            }
            frmDisplayImageOptions.ckbInvertYAxis.Checked = this.zoomPicBox1.InvertY;
            frmDisplayImageOptions.cbbInterpolation.SelectedIndex = (Int32)this.zoomPicBox1.InterpolationMode;
            if (frmDisplayImageOptions.ShowDialog() == DialogResult.OK)
            {
                this.zoomPicBox1.InterpolationMode = (InterpolationMode) frmDisplayImageOptions.cbbInterpolation.SelectedIndex;
                this.zoomPicBox1.InvertY = frmDisplayImageOptions.ckbInvertYAxis.Checked;
                this.zoomPicBox1.Invalidate();
            }
 

        }

         private void aboutToolStripMenuItem_Click_1(object sender, EventArgs e)
         {
             AboutBox1 frmAbout = new AboutBox1();
             frmAbout.ShowDialog();
  
         }

            
    }
} 